10   REM A simple XMODEM (with checksum) receive program for the Olivetti M20
20   REM The serial port must have been configured, put the 'ci' command
30   REM resident in memory with the 'pl ci' command in PCOS.
35   :
40   PP%=1                      ' Packet number counter
50   E%=0                       ' Receive error
60   PN%=0                      ' Port number (0 for com:)
70   C$=SPACE$(132)
75   :
80   PRINT "Olivetti M20 XMODEM receive program."
81   PRINT "Version 0.3, Davide Bucci 2005-2020"
85   INPUT "Enter file name: ", FI$
86   OPEN "a",1,FI$
87   PRINT "Prepare the uploader, then type a key to start transmission"
90   A$=INKEY$: IF A$="" THEN GOTO 90
95   PRINT "Transmission started"
96   GOSUB 2000                 ' Empty receive buffer
100  S$=CHR$(21) ' (NAK) Start transmission in checksum mode
101  E%=0
102  REM ----------
103  REM Main cycle
104  REM ----------
105  WHILE 1
106    CALL "ci"(PN%,"w",@E%, S$)   ' Send packet send command
107    IF E%<>0 THEN GOTO 5000  ' check for unrecoverable error
110    GOSUB 3000               ' receive a packet
120    IF ASC(MID$(C$, 1,1))<>1 THEN GOTO 1000 'check for SOH
130    KN%=ASC(MID$(C$,2,1))    ' read packet number
140    NP%=ASC(MID$(C$,3,1))    ' read 1-complement of packet number
150    IF KN%+NP%<>255 THEN GOTO 1000           ' they do not match
155    IF (PP% AND 255)<>KN% THEN GOTO 5000     ' packet number is unexpected
160    PK$=MID$(C$, 4, 128)     ' extract packet payload (128 bytes)
170    CK%=ASC(MID$(C$, 132, 1))' extract packet checksum
175    CC%=0                    ' calculate checksum of received data
180    FOR I%=1 TO 128
190      CC%=CC%+ASC(MID$(PK$,I%,1))
200    NEXT
210    IF (CC% AND 255) <> CK% THEN PRINT " Bad checksum ": GOTO 1000
215    PRINT ".";
216    PP%=PP%+1                ' increment packet number
217    PRINT #1, PK$;           ' write the received packet on output file
225    S$=CHR$(6)               ' send ACK (6) in the next iteration.
230  WEND
999  REM ------------------
1000 REM Transmission error
1001 REM ------------------
1002   PRINT " Transmission error"
1003   GOSUB 2000     ' empty buffer
1005   S$=CHR$(21)    ' send NAK
1010   GOTO 106
1998 :
1999 REM --------------------
2000 REM Empty receive buffer
2001 REM --------------------
2002   KK%=0
2010   HS%=0: DS%=0
2015   BC%=0
2020   CALL "ci"(PN%, "sr", @E%, @HS%, @DS%, @BC%) ' Get number of bytes avail.
2021   IF BC%<1 THEN GOTO 2040
2022   IF BC%>255 THEN BC%=255
2023   KK%=KK%+BC%
2025   P$=SPACE$(BC%)
2030   CALL "ci"(PN%, "r", @E%, P$, BC%)
2035   GOTO 2010
2040   PRINT "Removed ";KK%;"char(s) from the receive buffer"
2050   RETURN
2998 :
2999 REM ------------------------------
3000 REM Receive a packet, put it in C$
3002 REM ------------------------------
3003   P$=SPACE$(1)   ' Get number of bytes available in the receive buffer
3004   CALL "ci"(PN%, "sr", @E%, @HS%, @DS%, @BC%): IF E%<>0 THEN GOTO 5000
3005   IF BC%=0 THEN GOTO 3003 ELSE CALL "ci"(PB%, "r", @E%, P$,1) ' read 1 byte
3006   IF E%<>0 THEN GOTO 5000
3007   C$=""
3008   IF P$=CHR$(4) THEN GOTO 4000 'end of transmission
3010   C$=C$+P$   'IF P$=CHR$(1) THEN C$=C$+P$ ELSE GOTO 5000
3016   CALL "ci"(PN%, "sr", @E%, @HS%, @DS%, @BC%)
3017   IF BC%<131 THEN GOTO 3016  ' wait for the rest of the packet available
3018   IF E%<>0 THEN GOTO 5000
3019   P$=SPACE$(131)
3020   CALL "ci"(PN%, "r", @E%, P$,131)   ' read the rest of the packet
3021   IF E%<>0 THEN GOTO 5000
3030   C$=C$+P$
3040   RETURN
3998 :
3999 REM -------------------
4000 REM End of transmission
4001 REM -------------------
4005   CALL "ci"(PN%, "w", @E%, CHR$(21))             ' Send NAK (21)
4010   CALL "ci"(PN%, "sr", @E%, @HS%, @DS%, @BC%)    ' wait for an answer
4020   IF BC%<1 THEN GOTO 4010
4030   C$=SPACE$(1)
4040   CALL "ci"(PN%, "r", @E%,C$,1)                  ' read the answer
4050   IF C$<>CHR$(4) THEN GOTO 4000                  ' check for EOT (4)
4060   CALL "ci"(PN%, "w", @E%, CHR$(6))              ' Send
4065   PRINT
4070   PRINT "End of transmission"
4075   CLOSE 1
4080   END
4999 REM -------------------
5000 REM Unrecoverable error
5001 REM -------------------
5010   PRINT "Unrecoverable error"
5020   END